@@ -286,7 +286,7 @@ static GHashTable* _xfdashboard_application_tracker_get_environment_from_pid(gin
 		return(NULL);
 	}
 
-	/* Open enviroment variables of process.
+	/* Open environment variables of process.
 	 * This is the initial set of environment variables set when process was spawned.
 	 * But that is ok because the environment variables we lookup are set
 	 * at launch time and do not change.
@@ -298,7 +298,7 @@ static GHashTable* _xfdashboard_application_tracker_get_environment_from_pid(gin
 	if(!g_file_get_contents(procEnvFile, &envContent, &envLength, &error))
 	{
 		XFDASHBOARD_DEBUG(_xfdashboard_application_tracker, APPLICATIONS,
-							"Could not read enviroment varibles for PID %d at %s: %s",
+							"Could not read environment varibles for PID %d at %s: %s",
 							inPID,
 							procEnvFile,
 							error ? error->message : _("Unknown error"));
@@ -314,12 +314,12 @@ static GHashTable* _xfdashboard_application_tracker_get_environment_from_pid(gin
 	}
 
 	XFDASHBOARD_DEBUG(_xfdashboard_application_tracker, APPLICATIONS,
-						"Enviroment set for PID %d at %s is %lu bytes long",
+						"environment set for PID %d at %s is %lu bytes long",
 						inPID,
 						procEnvFile,
 						envLength);
 
-	/* Iterate through enviroment variables and insert copy of environment
+	/* Iterate through environment variables and insert copy of environment
 	 * variable's name as key and the copy of its value as value into hash-table.
 	 */
 	iter=envContent;
@@ -355,7 +355,7 @@ static GHashTable* _xfdashboard_application_tracker_get_environment_from_pid(gin
 		}
 		if(*iter!='=')
 		{
-			g_warning(_("Malformed enviroment '%s' in environment set for PID %d at %s"),
+			g_warning(_("Malformed environment '%s' in environment set for PID %d at %s"),
 						name,
 						inPID,
 						procEnvFile);
@@ -545,7 +545,7 @@ static GAppInfo* _xfdashboard_application_tracker_get_desktop_id_from_environmen
 	 * application database.
 	 */
 	XFDASHBOARD_DEBUG(self, APPLICATIONS,
-						"Resolved enviroment variables of window '%s' to desktop ID '%s'",
+						"Resolved environment variables of window '%s' to desktop ID '%s'",
 						xfdashboard_window_tracker_window_get_name(inWindow),
 						foundAppInfo ? g_app_info_get_id(foundAppInfo) : "<nil>");
 
@@ -931,6 +931,26 @@ static void _xfdashboard_application_tracker_on_active_window_changed(Xfdashboar
 
 /* IMPLEMENTATION: GObject */
 
+/* Construct this object */
+static GObject* _xfdashboard_application_tracker_constructor(GType inType,
+																guint inNumberConstructParams,
+																GObjectConstructParam *inConstructParams)
+{
+	GObject									*object;
+
+	if(!_xfdashboard_application_tracker)
+	{
+		object=G_OBJECT_CLASS(xfdashboard_application_tracker_parent_class)->constructor(inType, inNumberConstructParams, inConstructParams);
+		_xfdashboard_application_tracker=XFDASHBOARD_APPLICATION_TRACKER(object);
+	}
+		else
+		{
+			object=g_object_ref(G_OBJECT(_xfdashboard_application_tracker));
+		}
+
+	return(object);
+}
+
 /* Dispose this object */
 static void _xfdashboard_application_tracker_dispose(GObject *inObject)
 {
@@ -957,14 +977,21 @@ static void _xfdashboard_application_tracker_dispose(GObject *inObject)
 		priv->appDatabase=NULL;
 	}
 
-	/* Unset singleton */
+	/* Call parent's class dispose method */
+	G_OBJECT_CLASS(xfdashboard_application_tracker_parent_class)->dispose(inObject);
+}
+
+/* Finalize this object */
+static void _xfdashboard_application_tracker_finalize(GObject *inObject)
+{
+	/* Release allocated resources finally, e.g. unset singleton */
 	if(G_LIKELY(G_OBJECT(_xfdashboard_application_tracker)==inObject))
 	{
 		_xfdashboard_application_tracker=NULL;
 	}
 
 	/* Call parent's class dispose method */
-	G_OBJECT_CLASS(xfdashboard_application_tracker_parent_class)->dispose(inObject);
+	G_OBJECT_CLASS(xfdashboard_application_tracker_parent_class)->finalize(inObject);
 }
 
 /* Class initialization
@@ -976,7 +1003,9 @@ static void xfdashboard_application_tracker_class_init(XfdashboardApplicationTra
 	GObjectClass		*gobjectClass=G_OBJECT_CLASS(klass);
 
 	/* Override functions */
+	gobjectClass->constructor=_xfdashboard_application_tracker_constructor;
 	gobjectClass->dispose=_xfdashboard_application_tracker_dispose;
+	gobjectClass->finalize=_xfdashboard_application_tracker_finalize;
 
 	/* Set up private structure */
 	g_type_class_add_private(klass, sizeof(XfdashboardApplicationTrackerPrivate));
@@ -1036,13 +1065,10 @@ static void xfdashboard_application_tracker_init(XfdashboardApplicationTracker *
 /* Get single instance of application */
 XfdashboardApplicationTracker* xfdashboard_application_tracker_get_default(void)
 {
-	if(G_UNLIKELY(_xfdashboard_application_tracker==NULL))
-	{
-		_xfdashboard_application_tracker=g_object_new(XFDASHBOARD_TYPE_APPLICATION_TRACKER, NULL);
-	}
-		else g_object_ref(_xfdashboard_application_tracker);
+	GObject									*singleton;
 
-	return(_xfdashboard_application_tracker);
+	singleton=g_object_new(XFDASHBOARD_TYPE_APPLICATION_TRACKER, NULL);
+	return(XFDASHBOARD_APPLICATION_TRACKER(singleton));
 }
 
 /* Get running state of application */
