@@ -39,6 +39,21 @@ G_BEGIN_DECLS
 #define clutter_actor_get_reactive(actor)	CLUTTER_ACTOR_IS_REACTIVE( (actor) )
 #endif
 
+#if !GLIB_CHECK_VERSION(2, 44, 0)
+inline static gboolean g_strv_contains(const gchar * const *inStringList, const gchar *inString)
+{
+	g_return_val_if_fail(inStringList, FALSE);
+	g_return_val_if_fail(inString, FALSE);
+
+	for(; *inStringList; inStringList++)
+	{
+		if(g_str_equal(inString, *inStringList)) return(TRUE);
+	}
+
+	return(FALSE);
+}
+#endif
+
 G_END_DECLS
 
 #endif	/* __LIBXFDASHBOARD_COMPAT__ */
